call xerces.bat POReadAllXercesSax filename 1 
call xerces.bat POReadAllXercesSax filename 2 
call xerces.bat POReadAllXercesSax filename 3 
call xerces.bat POReadAllXercesSax filename 4 
call xerces.bat POReadAllXercesSax filename 5 
call xerces.bat POReadAllXercesSax filename 6 
call xerces.bat POReadAllXercesSax filename 7 
call xerces.bat POReadOneXercesSax filename 1 
call xerces.bat POReadOneXercesSax filename 2 
call xerces.bat POReadOneXercesSax filename 3 
call xerces.bat POReadOneXercesSax filename 4 
call xerces.bat POReadOneXercesSax filename 5 
call xerces.bat POReadOneXercesSax filename 6 
call xerces.bat POReadOneXercesSax filename 7 
call xerces.bat POTopDownXercesSax
call xerces.bat BisLoadXercesSax flavor deep-attributes
call xerces.bat BisLoadXercesSax flavor deep-elements
call xerces.bat BisLoadXercesSax flavor wide-attributes
call xerces.bat BisLoadXercesSax flavor wide-elements
call xerces.bat BisLoadXercesSax flavor wide-text
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 64 flavor deep-attributes
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 128 flavor deep-attributes
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 256 flavor deep-attributes
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 512 flavor deep-attributes
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 64 flavor deep-elements
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 128 flavor deep-elements
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 256 flavor deep-elements
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 512 flavor deep-elements
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 64 flavor wide-attributes
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 128 flavor wide-attributes
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 256 flavor wide-attributes
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 512 flavor wide-attributes
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 64 flavor wide-elements
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 128 flavor wide-elements
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 256 flavor wide-elements
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 512 flavor wide-elements
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 64 flavor wide-text
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 128 flavor wide-text
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 256 flavor wide-text
call xerces.bat MaxMemUntypedBisLoadXercesSax memory 512 flavor wide-text

call xerces.bat POValidateXerces

call xerces.bat POReadAllXercesDom filename 1 
call xerces.bat POReadAllXercesDom filename 2 
call xerces.bat POReadAllXercesDom filename 3 
call xerces.bat POReadAllXercesDom filename 4 
call xerces.bat POReadAllXercesDom filename 5 
call xerces.bat POReadAllXercesDom filename 6 
call xerces.bat POReadAllXercesDom filename 7 
call xerces.bat POReadOneXercesDom filename 1 
call xerces.bat POReadOneXercesDom filename 2 
call xerces.bat POReadOneXercesDom filename 3 
call xerces.bat POReadOneXercesDom filename 4 
call xerces.bat POReadOneXercesDom filename 5 
call xerces.bat POReadOneXercesDom filename 6 
call xerces.bat POReadOneXercesDom filename 7 
call xerces.bat POTopDownXercesDom
call xerces.bat BisLoadXercesDom flavor deep-attributes
call xerces.bat BisLoadXercesDom flavor deep-elements
call xerces.bat BisLoadXercesDom flavor wide-attributes
call xerces.bat BisLoadXercesDom flavor wide-elements
call xerces.bat BisLoadXercesDom flavor wide-text 
call xerces.bat DomWalkXerces flavor deep-attributes
call xerces.bat DomWalkXerces flavor deep-elements
call xerces.bat DomWalkXerces flavor wide-attributes
call xerces.bat DomWalkXerces flavor wide-elements
call xerces.bat DomWalkXerces flavor wide-text
call xerces.bat DomWalkNLXerces flavor deep-attributes
call xerces.bat DomWalkNLXerces flavor deep-elements
call xerces.bat DomWalkNLXerces flavor wide-attributes
call xerces.bat DomWalkNLXerces flavor wide-elements
call xerces.bat DomWalkNLXerces flavor wide-text
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 64 flavor deep-attributes
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 128 flavor deep-attributes
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 256 flavor deep-attributes
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 512 flavor deep-attributes
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 64 flavor deep-elements
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 128 flavor deep-elements
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 256 flavor deep-elements
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 512 flavor deep-elements
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 64 flavor wide-attributes
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 128 flavor wide-attributes
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 256 flavor wide-attributes
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 512 flavor wide-attributes
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 64 flavor wide-elements
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 128 flavor wide-elements
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 256 flavor wide-elements
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 512 flavor wide-elements
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 64 flavor wide-text
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 128 flavor wide-text
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 256 flavor wide-text
call xerces.bat MaxMemUntypedBisLoadXercesDom memory 512 flavor wide-text
