
/*
This Java source file was generated by test-to-java.xsl
and is a derived work from the source document.
The source document contained the following notice:


Copyright (c) 2001 World Wide Web Consortium,
(Massachusetts Institute of Technology, Institut National de
Recherche en Informatique et en Automatique, Keio University). All
Rights Reserved. This program is distributed under the W3C's Software
Intellectual Property License. This program is distributed in the
hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.
See W3C License http://www.w3.org/Consortium/Legal/ for more details.

*/

//package org.w3c.domts.level2.core;
package dom.detailed.w3c;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.w3c.domts.level2.core.*;


    /**
    *  Test suite
    *  @author W3C DOM Test Working Group
    **/
    public class ibmtests extends TestCase{



    public static Test suite() {
       TestSuite sink= new TestSuite();
     // sink.addTestSuite(attrgetownerelement01.class);
      sink.addTestSuite(attrgetownerelement02.class);
      sink.addTestSuite(attrgetownerelement03.class);
     // sink.addTestSuite(attrgetownerelement04.class);
      sink.addTestSuite(attrgetownerelement05.class);
      sink.addTestSuite(documentcreateattributeNS01.class);
      sink.addTestSuite(documentcreateattributeNS02.class);
      sink.addTestSuite(documentcreateattributeNS03.class);
      sink.addTestSuite(documentcreateattributeNS04.class);
      sink.addTestSuite(documentcreateattributeNS05.class);
      sink.addTestSuite(documentcreateattributeNS06.class);
      sink.addTestSuite(documentcreateattributeNS07.class);
      sink.addTestSuite(documentcreateelementNS01.class);
      sink.addTestSuite(documentcreateelementNS02.class);
      sink.addTestSuite(documentcreateelementNS05.class);
      sink.addTestSuite(documentcreateelementNS06.class);
      sink.addTestSuite(domimplementationcreatedocument03.class);
      sink.addTestSuite(domimplementationcreatedocument04.class);
      sink.addTestSuite(domimplementationcreatedocument05.class);
      sink.addTestSuite(domimplementationcreatedocument07.class);
      sink.addTestSuite(domimplementationcreatedocumenttype02.class);
      sink.addTestSuite(domimplementationcreatedocumenttype04.class);
      sink.addTestSuite(domimplementationhasfeature01.class);
      sink.addTestSuite(domimplementationhasfeature02.class);
  //    sink.addTestSuite(documentgetelementbyid01.class);
      sink.addTestSuite(documentgetelementsbytagnameNS01.class);
      sink.addTestSuite(documentgetelementsbytagnameNS02.class);
      sink.addTestSuite(documentgetelementsbytagnameNS03.class);
      sink.addTestSuite(documentgetelementsbytagnameNS04.class);
      sink.addTestSuite(documentgetelementsbytagnameNS05.class);
      sink.addTestSuite(documentimportnode01.class);
  //    sink.addTestSuite(documentimportnode02.class);
      sink.addTestSuite(documentimportnode03.class);
   //   sink.addTestSuite(documentimportnode04.class);
      sink.addTestSuite(documentimportnode05.class);
      sink.addTestSuite(documentimportnode06.class);
     // sink.addTestSuite(documentimportnode07.class);
      sink.addTestSuite(documentimportnode08.class);
      sink.addTestSuite(documentimportnode09.class);
   //   sink.addTestSuite(documentimportnode10.class);
      sink.addTestSuite(documentimportnode11.class);
      sink.addTestSuite(documentimportnode12.class);
      sink.addTestSuite(documentimportnode13.class);
      sink.addTestSuite(documentimportnode14.class);
      sink.addTestSuite(documentimportnode15.class);
      sink.addTestSuite(documentimportnode17.class);
      sink.addTestSuite(documentimportnode18.class);
   //   sink.addTestSuite(documentimportnode19.class);
    //  sink.addTestSuite(documentimportnode20.class);
    //  sink.addTestSuite(documentimportnode21.class);
     // sink.addTestSuite(documentimportnode22.class);
  //    sink.addTestSuite(documenttypeinternalSubset01.class);
   //   sink.addTestSuite(documenttypepublicid01.class);
   //   sink.addTestSuite(documenttypesystemid01.class);
      sink.addTestSuite(elementgetattributens02.class);
      sink.addTestSuite(elementgetattributenodens01.class);
      sink.addTestSuite(elementgetattributenodens02.class);
      sink.addTestSuite(elementgetattributenodens03.class);
      sink.addTestSuite(elementgetelementsbytagnamens02.class);
      sink.addTestSuite(elementgetelementsbytagnamens04.class);
      sink.addTestSuite(elementgetelementsbytagnamens05.class);
      sink.addTestSuite(elementhasattribute01.class);
      sink.addTestSuite(elementhasattribute02.class);
      sink.addTestSuite(elementhasattribute03.class);
      sink.addTestSuite(elementhasattribute04.class);
      sink.addTestSuite(elementhasattributens01.class);
      sink.addTestSuite(elementhasattributens02.class);
      sink.addTestSuite(elementhasattributens03.class);
      sink.addTestSuite(elementremoveattributens01.class);
      sink.addTestSuite(elementsetattributens01.class);
      sink.addTestSuite(elementsetattributens02.class);
      sink.addTestSuite(elementsetattributens03.class);
      sink.addTestSuite(elementsetattributens04.class);
      sink.addTestSuite(elementsetattributens05.class);
      sink.addTestSuite(elementsetattributens08.class);
      sink.addTestSuite(elementsetattributenodens01.class);
      sink.addTestSuite(elementsetattributenodens02.class);
      sink.addTestSuite(elementsetattributenodens03.class);
      sink.addTestSuite(elementsetattributenodens04.class);
      sink.addTestSuite(elementsetattributenodens05.class);
      sink.addTestSuite(elementsetattributenodens06.class);
      sink.addTestSuite(namednodemapgetnameditemns01.class);
      sink.addTestSuite(namednodemapgetnameditemns02.class);
      sink.addTestSuite(namednodemapgetnameditemns03.class);
      sink.addTestSuite(namednodemapgetnameditemns04.class);
      sink.addTestSuite(namednodemapgetnameditemns05.class);
      sink.addTestSuite(namednodemapgetnameditemns06.class);
      sink.addTestSuite(namednodemapsetnameditemns01.class);
      sink.addTestSuite(namednodemapsetnameditemns02.class);
      sink.addTestSuite(namednodemapsetnameditemns03.class);
      sink.addTestSuite(namednodemapsetnameditemns04.class);
      sink.addTestSuite(namednodemapsetnameditemns05.class);
      sink.addTestSuite(namednodemapsetnameditemns06.class);
      sink.addTestSuite(namednodemapsetnameditemns07.class);
      sink.addTestSuite(namednodemapsetnameditemns08.class);
      sink.addTestSuite(namednodemapsetnameditemns09.class);
      sink.addTestSuite(namednodemapsetnameditemns10.class);
      sink.addTestSuite(namednodemapremovenameditemns01.class);
      sink.addTestSuite(namednodemapremovenameditemns02.class);
      sink.addTestSuite(namednodemapremovenameditemns03.class);
      sink.addTestSuite(namednodemapremovenameditemns04.class);
      sink.addTestSuite(namednodemapremovenameditemns05.class);
      sink.addTestSuite(namednodemapremovenameditemns06.class);
      sink.addTestSuite(namednodemapremovenameditemns07.class);
      sink.addTestSuite(namednodemapremovenameditemns08.class);
      sink.addTestSuite(namednodemapremovenameditemns09.class);
      sink.addTestSuite(nodegetlocalname03.class);
      sink.addTestSuite(nodegetnamespaceuri03.class);
      sink.addTestSuite(nodegetownerdocument01.class);
      sink.addTestSuite(nodegetownerdocument02.class);
      sink.addTestSuite(nodegetprefix03.class);
      sink.addTestSuite(nodesetprefix01.class);
      sink.addTestSuite(nodesetprefix02.class);
      sink.addTestSuite(nodesetprefix03.class);
      sink.addTestSuite(nodesetprefix04.class);
      sink.addTestSuite(nodesetprefix05.class);
      sink.addTestSuite(nodesetprefix06.class);
      sink.addTestSuite(nodesetprefix07.class);
      sink.addTestSuite(nodesetprefix08.class);
      sink.addTestSuite(nodesetprefix09.class);
      sink.addTestSuite(nodehasattributes01.class);
      sink.addTestSuite(nodehasattributes02.class);
      sink.addTestSuite(nodehasattributes03.class);
      sink.addTestSuite(nodehasattributes04.class);
      sink.addTestSuite(nodeissupported01.class);
      sink.addTestSuite(nodeissupported02.class);
      sink.addTestSuite(nodeissupported03.class);
      sink.addTestSuite(nodeissupported04.class);
      sink.addTestSuite(nodeissupported05.class);

       return sink;
   }


}
