sh ./xerces.sh POReadAllXercesSax filename 1 
sh ./xerces.sh POReadAllXercesSax filename 2 
sh ./xerces.sh POReadAllXercesSax filename 3 
sh ./xerces.sh POReadAllXercesSax filename 4 
sh ./xerces.sh POReadAllXercesSax filename 5 
sh ./xerces.sh POReadAllXercesSax filename 6 
sh ./xerces.sh POReadAllXercesSax filename 7 
sh ./xerces.sh POReadOneXercesSax filename 1 
sh ./xerces.sh POReadOneXercesSax filename 2 
sh ./xerces.sh POReadOneXercesSax filename 3 
sh ./xerces.sh POReadOneXercesSax filename 4 
sh ./xerces.sh POReadOneXercesSax filename 5 
sh ./xerces.sh POReadOneXercesSax filename 6 
sh ./xerces.sh POReadOneXercesSax filename 7 
sh ./xerces.sh POTopDownXercesSax
sh ./xerces.sh BisLoadXercesSax flavor deep-attributes
sh ./xerces.sh BisLoadXercesSax flavor deep-elements
sh ./xerces.sh BisLoadXercesSax flavor wide-attributes
sh ./xerces.sh BisLoadXercesSax flavor wide-elements
sh ./xerces.sh BisLoadXercesSax flavor wide-text
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 64 flavor deep-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 128 flavor deep-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 256 flavor deep-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 512 flavor deep-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 64 flavor deep-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 128 flavor deep-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 256 flavor deep-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 512 flavor deep-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 64 flavor wide-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 128 flavor wide-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 256 flavor wide-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 512 flavor wide-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 64 flavor wide-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 128 flavor wide-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 256 flavor wide-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 512 flavor wide-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 64 flavor wide-text
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 128 flavor wide-text
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 256 flavor wide-text
sh ./xerces.sh MaxMemUntypedBisLoadXercesSax memory 512 flavor wide-text

sh ./xerces.sh POValidateXerces

sh ./xerces.sh POReadAllXercesDom filename 1 
sh ./xerces.sh POReadAllXercesDom filename 2 
sh ./xerces.sh POReadAllXercesDom filename 3 
sh ./xerces.sh POReadAllXercesDom filename 4 
sh ./xerces.sh POReadAllXercesDom filename 5 
sh ./xerces.sh POReadAllXercesDom filename 6 
sh ./xerces.sh POReadAllXercesDom filename 7 
sh ./xerces.sh POReadOneXercesDom filename 1 
sh ./xerces.sh POReadOneXercesDom filename 2 
sh ./xerces.sh POReadOneXercesDom filename 3 
sh ./xerces.sh POReadOneXercesDom filename 4 
sh ./xerces.sh POReadOneXercesDom filename 5 
sh ./xerces.sh POReadOneXercesDom filename 6 
sh ./xerces.sh POReadOneXercesDom filename 7 
sh ./xerces.sh POTopDownXercesDom
sh ./xerces.sh BisLoadXercesDom flavor deep-attributes
sh ./xerces.sh BisLoadXercesDom flavor deep-elements
sh ./xerces.sh BisLoadXercesDom flavor wide-attributes
sh ./xerces.sh BisLoadXercesDom flavor wide-elements
sh ./xerces.sh BisLoadXercesDom flavor wide-text 
sh ./xerces.sh DomWalkXerces flavor deep-attributes
sh ./xerces.sh DomWalkXerces flavor deep-elements
sh ./xerces.sh DomWalkXerces flavor wide-attributes
sh ./xerces.sh DomWalkXerces flavor wide-elements
sh ./xerces.sh DomWalkXerces flavor wide-text
sh ./xerces.sh DomWalkNLXerces flavor deep-attributes
sh ./xerces.sh DomWalkNLXerces flavor deep-elements
sh ./xerces.sh DomWalkNLXerces flavor wide-attributes
sh ./xerces.sh DomWalkNLXerces flavor wide-elements
sh ./xerces.sh DomWalkNLXerces flavor wide-text
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 64 flavor deep-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 128 flavor deep-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 256 flavor deep-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 512 flavor deep-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 64 flavor deep-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 128 flavor deep-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 256 flavor deep-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 512 flavor deep-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 64 flavor wide-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 128 flavor wide-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 256 flavor wide-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 512 flavor wide-attributes
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 64 flavor wide-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 128 flavor wide-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 256 flavor wide-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 512 flavor wide-elements
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 64 flavor wide-text
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 128 flavor wide-text
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 256 flavor wide-text
sh ./xerces.sh MaxMemUntypedBisLoadXercesDom memory 512 flavor wide-text
