/*   Copyright 2004 The Apache Software Foundation
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package tools.inst2xsd.checkin;

import tools.inst2xsd.common.Inst2XsdTestBase;
import tools.xml.XmlComparator;
import org.apache.xmlbeans.impl.inst2xsd.Inst2XsdOptions;
import org.apache.xmlbeans.impl.inst2xsd.Inst2Xsd;


import java.io.File;
import java.io.IOException;

import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.impl.xb.xsdschema.SchemaDocument;

public class Inst2XsdOptionsTest extends Inst2XsdTestBase {

    public Inst2XsdOptionsTest(String name) {
        super(name);
    }

    /** Move tests from Detailed back once cursor bug gets fixed */
    public void test_move() throws Exception{
        System.out.println("Move tests from Detailed back once cursor bug gets fixed");
    }












}
