/*   Copyright 2004 The Apache Software Foundation
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package scomp.derivation.detailed;

/**
 *
 *
 *
 */

/**
 * Testing final
 * This is a compile time test, not a runtime one
 * since final determines whether a type can be extended or not
 */
public class FinalTest {

    public void testFinalAll() throws Throwable {
    }

    public void testFinalExtension() throws Throwable {
    }

    public void testFinalRestriction() throws Throwable {
    }

    //should be equivaluent to final="#all"
    public void testFinalRE_ER() throws Throwable {
    }

    public void testFinalFinalDefault() throws Throwable {
    }

    public void testFinalNone() throws Throwable {
    }

}
