sh ./castor.sh POReadAllCastor 1 
sh ./castor.sh POReadAllCastor 2 
sh ./castor.sh POReadAllCastor 3 
sh ./castor.sh POReadAllCastor 4 
sh ./castor.sh POReadAllCastor 5 
sh ./castor.sh POReadAllCastor 6 
sh ./castor.sh POReadAllCastor 7 
sh ./castor.sh POReadOneCastor 1 
sh ./castor.sh POReadOneCastor 2 
sh ./castor.sh POReadOneCastor 3 
sh ./castor.sh POReadOneCastor 4 
sh ./castor.sh POReadOneCastor 5 
sh ./castor.sh POReadOneCastor 6 
sh ./castor.sh POReadOneCastor 7 
sh ./castor.sh POGetCustNameCastor filename 1
sh ./castor.sh POGetCustNameCastor filename 2
sh ./castor.sh POGetCustNameCastor filename 3
sh ./castor.sh POGetCustNameCastor filename 4
sh ./castor.sh POGetCustNameCastor filename 5
sh ./castor.sh POGetCustNameCastor filename 6
sh ./castor.sh POGetCustNameCastor filename 7
sh ./castor.sh POGetSetGetCustNameCastor filename 1
sh ./castor.sh POGetSetGetCustNameCastor filename 2
sh ./castor.sh POGetSetGetCustNameCastor filename 3
sh ./castor.sh POGetSetGetCustNameCastor filename 4
sh ./castor.sh POGetSetGetCustNameCastor filename 5
sh ./castor.sh POGetSetGetCustNameCastor filename 6
sh ./castor.sh POGetSetGetCustNameCastor filename 7
sh ./castor.sh POTopDownCastor
sh ./castor.sh POTopDownSaveCastor
sh ./castor.sh PrimTopDownCastor
sh ./castor.sh NPrimTopDownCastor
