call jaxb.bat POReadAllJaxb filename 1 
call jaxb.bat POReadAllJaxb filename 2 
call jaxb.bat POReadAllJaxb filename 3 
call jaxb.bat POReadAllJaxb filename 4 
call jaxb.bat POReadAllJaxb filename 5 
call jaxb.bat POReadAllJaxb filename 6 
call jaxb.bat POReadAllJaxb filename 7 
call jaxb.bat POReadOneJaxb filename 1
call jaxb.bat POReadOneJaxb filename 2
call jaxb.bat POReadOneJaxb filename 3
call jaxb.bat POReadOneJaxb filename 4
call jaxb.bat POReadOneJaxb filename 5
call jaxb.bat POReadOneJaxb filename 6
call jaxb.bat POReadOneJaxb filename 7
call jaxb.bat POGetCustNameJaxb filename 1
call jaxb.bat POGetCustNameJaxb filename 2
call jaxb.bat POGetCustNameJaxb filename 3
call jaxb.bat POGetCustNameJaxb filename 4
call jaxb.bat POGetCustNameJaxb filename 5
call jaxb.bat POGetCustNameJaxb filename 6
call jaxb.bat POGetCustNameJaxb filename 7
call jaxb.bat POGetSetGetCustNameJaxb filename 1
call jaxb.bat POGetSetGetCustNameJaxb filename 2
call jaxb.bat POGetSetGetCustNameJaxb filename 3
call jaxb.bat POGetSetGetCustNameJaxb filename 4
call jaxb.bat POGetSetGetCustNameJaxb filename 5
call jaxb.bat POGetSetGetCustNameJaxb filename 6
call jaxb.bat POGetSetGetCustNameJaxb filename 7
call jaxb.bat POTopDownJaxb
call jaxb.bat POTopDownSaveJaxb
call jaxb.bat PrimTopDownJaxb
call jaxb.bat NPrimTopDownJaxb
