/*   Copyright 2004 The Apache Software Foundation
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.apache.xmlbeans;

import org.apache.xmlbeans.xml.stream.XMLStreamException;

/**
 * An exception thrown from a validating XMLInputStream.
 * 
 * @see XmlObject.Factory#newValidatingXMLInputStream
 * @see SchemaTypeLoader#newValidatingXMLInputStream 
 */
public class XMLStreamValidationException extends XMLStreamException
{
    public XMLStreamValidationException ( XmlError xmlError )
    {
        super( xmlError.toString() );
        
        _xmlError = xmlError;
    }

    public XmlError getXmlError ( )
    {
        return _xmlError;
    }

    private XmlError _xmlError;
}