sh ./jaxb.sh POReadAllJaxb 1 
sh ./jaxb.sh POReadAllJaxb 2 
sh ./jaxb.sh POReadAllJaxb 3 
sh ./jaxb.sh POReadAllJaxb 4 
sh ./jaxb.sh POReadAllJaxb 5 
sh ./jaxb.sh POReadAllJaxb 6 
sh ./jaxb.sh POReadAllJaxb 7 
sh ./jaxb.sh POReadOneJaxb 1 
sh ./jaxb.sh POReadOneJaxb 2 
sh ./jaxb.sh POReadOneJaxb 3 
sh ./jaxb.sh POReadOneJaxb 4 
sh ./jaxb.sh POReadOneJaxb 5 
sh ./jaxb.sh POReadOneJaxb 6 
sh ./jaxb.sh POReadOneJaxb 7 
sh ./jaxb.sh POGetCustNameJaxb filename 1
sh ./jaxb.sh POGetCustNameJaxb filename 2
sh ./jaxb.sh POGetCustNameJaxb filename 3
sh ./jaxb.sh POGetCustNameJaxb filename 4
sh ./jaxb.sh POGetCustNameJaxb filename 5
sh ./jaxb.sh POGetCustNameJaxb filename 6
sh ./jaxb.sh POGetCustNameJaxb filename 7
sh ./jaxb.sh POGetSetGetCustNameJaxb filename 1
sh ./jaxb.sh POGetSetGetCustNameJaxb filename 2
sh ./jaxb.sh POGetSetGetCustNameJaxb filename 3
sh ./jaxb.sh POGetSetGetCustNameJaxb filename 4
sh ./jaxb.sh POGetSetGetCustNameJaxb filename 5
sh ./jaxb.sh POGetSetGetCustNameJaxb filename 6
sh ./jaxb.sh POGetSetGetCustNameJaxb filename 7
sh ./jaxb.sh POTopDownJaxb
sh ./jaxb.sh POTopDownSaveJaxb
sh ./jaxb.sh PrimTopDownJaxb
sh ./jaxb.sh NPrimTopDownJaxb
