call v1.bat POReadAllV1 filename 1 
call v1.bat POReadAllV1 filename 2 
call v1.bat POReadAllV1 filename 3 
call v1.bat POReadAllV1 filename 4 
call v1.bat POReadAllV1 filename 5 
call v1.bat POReadAllV1 filename 6 
call v1.bat POReadAllV1 filename 7 
call v1.bat POReadOneV1 filename 1
call v1.bat POReadOneV1 filename 2
call v1.bat POReadOneV1 filename 3
call v1.bat POReadOneV1 filename 4
call v1.bat POReadOneV1 filename 5
call v1.bat POReadOneV1 filename 6
call v1.bat POReadOneV1 filename 7
call v1.bat POGetCustNameV1 filename 1
call v1.bat POGetCustNameV1 filename 2
call v1.bat POGetCustNameV1 filename 3
call v1.bat POGetCustNameV1 filename 4
call v1.bat POGetCustNameV1 filename 5
call v1.bat POGetCustNameV1 filename 6
call v1.bat POGetCustNameV1 filename 7
call v1.bat POGetSetGetCustNameV1 filename 1
call v1.bat POGetSetGetCustNameV1 filename 2
call v1.bat POGetSetGetCustNameV1 filename 3
call v1.bat POGetSetGetCustNameV1 filename 4
call v1.bat POGetSetGetCustNameV1 filename 5
call v1.bat POGetSetGetCustNameV1 filename 6
call v1.bat POGetSetGetCustNameV1 filename 7
call v1.bat POSelectLineItemsV1 filename 1
call v1.bat POSelectLineItemsV1 filename 2
call v1.bat POSelectLineItemsV1 filename 3
call v1.bat POSelectLineItemsV1 filename 4
call v1.bat POSelectLineItemsV1 filename 5
call v1.bat POSelectLineItemsV1 filename 6
call v1.bat POSelectLineItemsV1 filename 7
call v1.bat POTopDownV1
call v1.bat POTopDownSaveV1
call v1.bat POValidateXmlObjV1
call v1.bat PrimTopDownV1
call v1.bat NPrimTopDownV1
call v1.bat CursorWalkV1 flavor deep-attributes
call v1.bat CursorWalkV1 flavor deep-elements
call v1.bat CursorWalkV1 flavor wide-attributes
call v1.bat CursorWalkV1 flavor wide-elements
call v1.bat CursorWalkV1 flavor wide-text
call v1.bat CursorGetAttributeV1 flavor deep-attributes
call v1.bat CursorGetAttributeV1 flavor wide-attributes
call v1.bat CursorGetSetGetAttributeV1 flavor deep-attributes
call v1.bat CursorGetSetGetAttributeV1 flavor wide-attributes
call v1.bat CursorGetElementV1
call v1.bat CursorGetSetGetElementV1
call v1.bat BisLoadV1 flavor deep-attributes
call v1.bat BisLoadV1 flavor deep-elements
call v1.bat BisLoadV1 flavor wide-attributes
call v1.bat BisLoadV1 flavor wide-elements
call v1.bat BisLoadV1 flavor wide-text
call v1.bat DomWalkV1 flavor deep-attributes
call v1.bat DomWalkV1 flavor deep-elements
call v1.bat DomWalkV1 flavor wide-attributes
call v1.bat DomWalkV1 flavor wide-elements
call v1.bat DomWalkV1 flavor wide-text
call v1.bat DomWalkNLV1 flavor deep-attributes
call v1.bat DomWalkNLV1 flavor deep-elements
call v1.bat DomWalkNLV1 flavor wide-attributes
call v1.bat DomWalkNLV1 flavor wide-elements
call v1.bat DomWalkNLV1 flavor wide-text
call v1.bat MaxMemUntypedBisLoadV1 memory 64 flavor deep-attributes
call v1.bat MaxMemUntypedBisLoadV1 memory 128 flavor deep-attributes
call v1.bat MaxMemUntypedBisLoadV1 memory 256 flavor deep-attributes
call v1.bat MaxMemUntypedBisLoadV1 memory 512 flavor deep-attributes
call v1.bat MaxMemUntypedBisLoadV1 memory 64 flavor deep-elements
call v1.bat MaxMemUntypedBisLoadV1 memory 128 flavor deep-elements
call v1.bat MaxMemUntypedBisLoadV1 memory 256 flavor deep-elements
call v1.bat MaxMemUntypedBisLoadV1 memory 512 flavor deep-elements
call v1.bat MaxMemUntypedBisLoadV1 memory 64 flavor wide-attributes
call v1.bat MaxMemUntypedBisLoadV1 memory 128 flavor wide-attributes
call v1.bat MaxMemUntypedBisLoadV1 memory 256 flavor wide-attributes
call v1.bat MaxMemUntypedBisLoadV1 memory 512 flavor wide-attributes
call v1.bat MaxMemUntypedBisLoadV1 memory 64 flavor wide-text
call v1.bat MaxMemUntypedBisLoadV1 memory 128 flavor wide-text
call v1.bat MaxMemUntypedBisLoadV1 memory 256 flavor wide-text
call v1.bat MaxMemUntypedBisLoadV1 memory 512 flavor wide-text
call v1.bat MaxMemUntypedBisLoadV1 memory 64 flavor wide-elements
call v1.bat MaxMemUntypedBisLoadV1 memory 128 flavor wide-elements
call v1.bat MaxMemUntypedBisLoadV1 memory 256 flavor wide-elements
call v1.bat MaxMemUntypedBisLoadV1 memory 512 flavor wide-elements
