call v2.bat POReadAllV2 filename 1 
call v2.bat POReadAllV2 filename 2 
call v2.bat POReadAllV2 filename 3 
call v2.bat POReadAllV2 filename 4 
call v2.bat POReadAllV2 filename 5 
call v2.bat POReadAllV2 filename 6 
call v2.bat POReadAllV2 filename 7 
call v2.bat POReadOneV2 filename 1
call v2.bat POReadOneV2 filename 2
call v2.bat POReadOneV2 filename 3
call v2.bat POReadOneV2 filename 4
call v2.bat POReadOneV2 filename 5
call v2.bat POReadOneV2 filename 6
call v2.bat POReadOneV2 filename 7
call v2.bat POGetCustNameV2 filename 1
call v2.bat POGetCustNameV2 filename 2
call v2.bat POGetCustNameV2 filename 3
call v2.bat POGetCustNameV2 filename 4
call v2.bat POGetCustNameV2 filename 5
call v2.bat POGetCustNameV2 filename 6
call v2.bat POGetCustNameV2 filename 7
call v2.bat POGetSetGetCustNameV2 filename 1
call v2.bat POGetSetGetCustNameV2 filename 2
call v2.bat POGetSetGetCustNameV2 filename 3
call v2.bat POGetSetGetCustNameV2 filename 4
call v2.bat POGetSetGetCustNameV2 filename 5
call v2.bat POGetSetGetCustNameV2 filename 6
call v2.bat POGetSetGetCustNameV2 filename 7
call v2.bat POSelectLineItemsV2 filename 1
call v2.bat POSelectLineItemsV2 filename 2
call v2.bat POSelectLineItemsV2 filename 3
call v2.bat POSelectLineItemsV2 filename 4
call v2.bat POSelectLineItemsV2 filename 5
call v2.bat POSelectLineItemsV2 filename 6
call v2.bat POSelectLineItemsV2 filename 7
call v2.bat POQueryLineItemsV2 filename 1
call v2.bat POQueryLineItemsV2 filename 2
call v2.bat POQueryLineItemsV2 filename 3
call v2.bat POQueryLineItemsV2 filename 4
call v2.bat POQueryLineItemsV2 filename 5
call v2.bat POQueryLineItemsV2 filename 6
call v2.bat POQueryLineItemsV2 filename 7
call v2.bat POTopDownV2
call v2.bat POTopDownSaveV2
call v2.bat POValidateXmlObjV2
call v2.bat PrimTopDownV2
call v2.bat NPrimTopDownV2
call v2.bat CursorWalkV2 flavor deep-attributes
call v2.bat CursorWalkV2 flavor deep-elements
call v2.bat CursorWalkV2 flavor wide-attributes
call v2.bat CursorWalkV2 flavor wide-elements
call v2.bat CursorWalkV2 flavor wide-text
call v2.bat CursorGetAttributeV2 flavor deep-attributes
call v2.bat CursorGetAttributeV2 flavor wide-attributes
call v2.bat CursorGetSetGetAttributeV2 flavor deep-attributes
call v2.bat CursorGetSetGetAttributeV2 flavor wide-attributes
call v2.bat CursorGetElementV2
call v2.bat CursorGetSetGetElementV2
call v2.bat BisLoadV2 flavor deep-attributes
call v2.bat BisLoadV2 flavor deep-elements
call v2.bat BisLoadV2 flavor wide-attributes
call v2.bat BisLoadV2 flavor wide-elements
call v2.bat BisLoadV2 flavor wide-text
call v2.bat DomWalkV2 flavor deep-attributes
call v2.bat DomWalkV2 flavor deep-elements
call v2.bat DomWalkV2 flavor wide-attributes
call v2.bat DomWalkV2 flavor wide-elements
call v2.bat DomWalkV2 flavor wide-text
call v2.bat DomWalkNLV2 flavor deep-attributes
call v2.bat DomWalkNLV2 flavor deep-elements
call v2.bat DomWalkNLV2 flavor wide-attributes
call v2.bat DomWalkNLV2 flavor wide-elements
call v2.bat DomWalkNLV2 flavor wide-text
