call svn.bat POReadAllSVN filename 1 
call svn.bat POReadAllSVN filename 2 
call svn.bat POReadAllSVN filename 3 
call svn.bat POReadAllSVN filename 4 
call svn.bat POReadAllSVN filename 5 
call svn.bat POReadAllSVN filename 6 
call svn.bat POReadAllSVN filename 7 
call svn.bat POReadOneSVN filename 1
call svn.bat POReadOneSVN filename 2
call svn.bat POReadOneSVN filename 3
call svn.bat POReadOneSVN filename 4
call svn.bat POReadOneSVN filename 5
call svn.bat POReadOneSVN filename 6
call svn.bat POReadOneSVN filename 7
call svn.bat POGetCustNameSVN filename 1
call svn.bat POGetCustNameSVN filename 2
call svn.bat POGetCustNameSVN filename 3
call svn.bat POGetCustNameSVN filename 4
call svn.bat POGetCustNameSVN filename 5
call svn.bat POGetCustNameSVN filename 6
call svn.bat POGetCustNameSVN filename 7
call svn.bat POGetSetGetCustNameSVN filename 1
call svn.bat POGetSetGetCustNameSVN filename 2
call svn.bat POGetSetGetCustNameSVN filename 3
call svn.bat POGetSetGetCustNameSVN filename 4
call svn.bat POGetSetGetCustNameSVN filename 5
call svn.bat POGetSetGetCustNameSVN filename 6
call svn.bat POGetSetGetCustNameSVN filename 7
call svn.bat POSelectLineItemsSVN filename 1
call svn.bat POSelectLineItemsSVN filename 2
call svn.bat POSelectLineItemsSVN filename 3
call svn.bat POSelectLineItemsSVN filename 4
call svn.bat POSelectLineItemsSVN filename 5
call svn.bat POSelectLineItemsSVN filename 6
call svn.bat POSelectLineItemsSVN filename 7
call svn.bat POQueryLineItemsSVN filename 1
call svn.bat POQueryLineItemsSVN filename 2
call svn.bat POQueryLineItemsSVN filename 3
call svn.bat POQueryLineItemsSVN filename 4
call svn.bat POQueryLineItemsSVN filename 5
call svn.bat POQueryLineItemsSVN filename 6
call svn.bat POQueryLineItemsSVN filename 7
call svn.bat POTopDownSVN
call svn.bat POTopDownSaveSVN
call svn.bat POValidateXmlObjSVN
call svn.bat PrimTopDownSVN
call svn.bat NPrimTopDownSVN
call svn.bat CursorWalkSVN flavor deep-attributes
call svn.bat CursorWalkSVN flavor deep-elements
call svn.bat CursorWalkSVN flavor wide-attributes
call svn.bat CursorWalkSVN flavor wide-elements
call svn.bat CursorWalkSVN flavor wide-text
call svn.bat CursorGetAttributeSVN flavor deep-attributes
call svn.bat CursorGetAttributeSVN flavor wide-attributes
call svn.bat CursorGetSetGetAttributeSVN flavor deep-attributes
call svn.bat CursorGetSetGetAttributeSVN flavor wide-attributes
call svn.bat CursorGetElementSVN
call svn.bat CursorGetSetGetElementSVN
call svn.bat BisLoadSVN flavor deep-attributes
call svn.bat BisLoadSVN flavor deep-elements
call svn.bat BisLoadSVN flavor wide-attributes
call svn.bat BisLoadSVN flavor wide-elements
call svn.bat BisLoadSVN flavor wide-text
call svn.bat DomWalkSVN flavor deep-attributes
call svn.bat DomWalkSVN flavor deep-elements
call svn.bat DomWalkSVN flavor wide-attributes
call svn.bat DomWalkSVN flavor wide-elements
call svn.bat DomWalkSVN flavor wide-text
call svn.bat DomWalkNLSVN flavor deep-attributes
call svn.bat DomWalkNLSVN flavor deep-elements
call svn.bat DomWalkNLSVN flavor wide-attributes
call svn.bat DomWalkNLSVN flavor wide-elements
call svn.bat DomWalkNLSVN flavor wide-text
call svn.bat MaxMemUntypedBisLoadSVN memory 64 flavor deep-attributes
call svn.bat MaxMemUntypedBisLoadSVN memory 128 flavor deep-attributes
call svn.bat MaxMemUntypedBisLoadSVN memory 256 flavor deep-attributes
call svn.bat MaxMemUntypedBisLoadSVN memory 512 flavor deep-attributes
call svn.bat MaxMemUntypedBisLoadSVN memory 64 flavor deep-elements
call svn.bat MaxMemUntypedBisLoadSVN memory 128 flavor deep-elements
call svn.bat MaxMemUntypedBisLoadSVN memory 256 flavor deep-elements
call svn.bat MaxMemUntypedBisLoadSVN memory 512 flavor deep-elements
call svn.bat MaxMemUntypedBisLoadSVN memory 64 flavor wide-attributes
call svn.bat MaxMemUntypedBisLoadSVN memory 128 flavor wide-attributes
call svn.bat MaxMemUntypedBisLoadSVN memory 256 flavor wide-attributes
call svn.bat MaxMemUntypedBisLoadSVN memory 512 flavor wide-attributes
call svn.bat MaxMemUntypedBisLoadSVN memory 64 flavor wide-elements
call svn.bat MaxMemUntypedBisLoadSVN memory 128 flavor wide-elements
call svn.bat MaxMemUntypedBisLoadSVN memory 256 flavor wide-elements
call svn.bat MaxMemUntypedBisLoadSVN memory 512 flavor wide-elements
call svn.bat MaxMemUntypedBisLoadSVN memory 64 flavor wide-text
call svn.bat MaxMemUntypedBisLoadSVN memory 128 flavor wide-text
call svn.bat MaxMemUntypedBisLoadSVN memory 256 flavor wide-text
call svn.bat MaxMemUntypedBisLoadSVN memory 512 flavor wide-text
