
/*
This Java source file was generated by test-to-java.xsl
and is a derived work from the source document.
The source document contained the following notice:


Copyright (c) 2001-2003 World Wide Web Consortium,
(Massachusetts Institute of Technology, Institut National de
Recherche en Informatique et en Automatique, Keio University). All
Rights Reserved. This program is distributed under the W3C's Software
Intellectual Property License. This program is distributed in the
hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.
See W3C License http://www.w3.org/Consortium/Legal/ for more details.

*/

//package org.w3c.domts.level2.core;
package dom.detailed.w3c;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.w3c.domts.level2.core.*;


    /**
    *  Test suite
    *  @author W3C DOM Test Working Group
    **/
    public class alltests extends TestCase{

   public static Test suite() {
     TestSuite sink= new TestSuite();
    //  sink.addTestSuite(attrgetownerelement01.class);
      sink.addTestSuite(attrgetownerelement02.class);
      sink.addTestSuite(attrgetownerelement03.class);
    //  sink.addTestSuite(attrgetownerelement04.class);
      sink.addTestSuite(attrgetownerelement05.class);
      sink.addTestSuite(createAttributeNS01.class);
      sink.addTestSuite(createAttributeNS02.class);
      sink.addTestSuite(createAttributeNS03.class);
      sink.addTestSuite(createAttributeNS04.class);
      sink.addTestSuite(createAttributeNS05.class);
      sink.addTestSuite(createDocument01.class);
      sink.addTestSuite(createDocument02.class);
  //    sink.addTestSuite(createDocument03.class);
    //  sink.addTestSuite(createDocument04.class);
      sink.addTestSuite(createDocument05.class);
      sink.addTestSuite(createDocument06.class);
      sink.addTestSuite(createDocument07.class);
   //   sink.addTestSuite(createDocumentType01.class);
    //  sink.addTestSuite(createDocumentType02.class);
   //   sink.addTestSuite(createDocumentType03.class);
      sink.addTestSuite(createElementNS01.class);
      sink.addTestSuite(createElementNS02.class);
      sink.addTestSuite(createElementNS03.class);
      sink.addTestSuite(createElementNS04.class);
      sink.addTestSuite(createElementNS05.class);
      sink.addTestSuite(documentcreateattributeNS01.class);
      sink.addTestSuite(documentcreateattributeNS02.class);
      sink.addTestSuite(documentcreateattributeNS03.class);
      sink.addTestSuite(documentcreateattributeNS04.class);
      sink.addTestSuite(documentcreateattributeNS05.class);
      sink.addTestSuite(documentcreateattributeNS06.class);
      sink.addTestSuite(documentcreateattributeNS07.class);
      sink.addTestSuite(documentcreateelementNS01.class);
      sink.addTestSuite(documentcreateelementNS02.class);
      sink.addTestSuite(documentcreateelementNS05.class);
      sink.addTestSuite(documentcreateelementNS06.class);
    //  sink.addTestSuite(documentgetelementbyid01.class);
      sink.addTestSuite(documentgetelementsbytagnameNS01.class);
      sink.addTestSuite(documentgetelementsbytagnameNS02.class);
      sink.addTestSuite(documentgetelementsbytagnameNS03.class);
      sink.addTestSuite(documentgetelementsbytagnameNS04.class);
      sink.addTestSuite(documentgetelementsbytagnameNS05.class);
      sink.addTestSuite(documentimportnode01.class);
     // sink.addTestSuite(documentimportnode02.class);
     // sink.addTestSuite(documentimportnode03.class);
    //  sink.addTestSuite(documentimportnode04.class);
      sink.addTestSuite(documentimportnode05.class);
      sink.addTestSuite(documentimportnode06.class);
    //  sink.addTestSuite(documentimportnode07.class);
    //  sink.addTestSuite(documentimportnode08.class);
      sink.addTestSuite(documentimportnode09.class);
      sink.addTestSuite(documentimportnode10.class);
      sink.addTestSuite(documentimportnode11.class);
      sink.addTestSuite(documentimportnode12.class);
      sink.addTestSuite(documentimportnode13.class);
      sink.addTestSuite(documentimportnode14.class);
      sink.addTestSuite(documentimportnode15.class);
      sink.addTestSuite(documentimportnode17.class);
      sink.addTestSuite(documentimportnode18.class);
     // sink.addTestSuite(documentimportnode19.class);
     // sink.addTestSuite(documentimportnode20.class);
     // sink.addTestSuite(documentimportnode21.class);
    //  sink.addTestSuite(documentimportnode22.class);
    //  sink.addTestSuite(documenttypeinternalSubset01.class);
     // sink.addTestSuite(documenttypepublicid01.class);
    //  sink.addTestSuite(documenttypesystemid01.class);
      sink.addTestSuite(domimplementationcreatedocument03.class);
      sink.addTestSuite(domimplementationcreatedocument04.class);
      sink.addTestSuite(domimplementationcreatedocument05.class);
      sink.addTestSuite(domimplementationcreatedocument07.class);
   //   sink.addTestSuite(domimplementationcreatedocumenttype01.class);
     // sink.addTestSuite(domimplementationcreatedocumenttype02.class);
    //  sink.addTestSuite(domimplementationcreatedocumenttype04.class);
     // sink.addTestSuite(domimplementationfeaturecore.class);
    //  sink.addTestSuite(domimplementationfeaturexmlversion2.class);
    //  sink.addTestSuite(domimplementationhasfeature01.class);
    //  sink.addTestSuite(domimplementationhasfeature02.class);
      sink.addTestSuite(elementgetattributenodens01.class);
      sink.addTestSuite(elementgetattributenodens02.class);
      // sink.addTestSuite(elementgetattributenodens03.class);
      // sink.addTestSuite(elementgetattributens02.class);
      sink.addTestSuite(elementgetelementsbytagnamens02.class);
      sink.addTestSuite(elementgetelementsbytagnamens04.class);
      sink.addTestSuite(elementgetelementsbytagnamens05.class);
      sink.addTestSuite(elementhasattribute01.class);
   //   sink.addTestSuite(elementhasattribute02.class);
      sink.addTestSuite(elementhasattribute03.class);
      sink.addTestSuite(elementhasattribute04.class);
      sink.addTestSuite(elementhasattributens01.class);
      sink.addTestSuite(elementhasattributens02.class);
      sink.addTestSuite(elementhasattributens03.class);
      sink.addTestSuite(elementremoveattributens01.class);
      sink.addTestSuite(elementsetattributenodens01.class);
      sink.addTestSuite(elementsetattributenodens02.class);
      sink.addTestSuite(elementsetattributenodens03.class);
      sink.addTestSuite(elementsetattributenodens04.class);
      sink.addTestSuite(elementsetattributenodens05.class);
    //  sink.addTestSuite(elementsetattributenodens06.class);
      sink.addTestSuite(elementsetattributens01.class);
      sink.addTestSuite(elementsetattributens02.class);
      sink.addTestSuite(elementsetattributens03.class);
      sink.addTestSuite(elementsetattributens04.class);
      sink.addTestSuite(elementsetattributens05.class);
      sink.addTestSuite(elementsetattributens08.class);
      sink.addTestSuite(elementsetattributensurinull.class);
     // sink.addTestSuite(getAttributeNS01.class);
      sink.addTestSuite(getAttributeNS02.class);
      sink.addTestSuite(getAttributeNS03.class);
      sink.addTestSuite(getAttributeNS04.class);
      sink.addTestSuite(getAttributeNS05.class);
      sink.addTestSuite(getAttributeNodeNS01.class);
      sink.addTestSuite(getAttributeNodeNS02.class);
    //  sink.addTestSuite(getElementById01.class);
     // sink.addTestSuite(getElementById02.class);
    //  sink.addTestSuite(getElementsByTagNameNS01.class);
      sink.addTestSuite(getElementsByTagNameNS02.class);
      sink.addTestSuite(getElementsByTagNameNS03.class);
      sink.addTestSuite(getElementsByTagNameNS04.class);
      sink.addTestSuite(getElementsByTagNameNS05.class);
      sink.addTestSuite(getElementsByTagNameNS06.class);
      sink.addTestSuite(getElementsByTagNameNS07.class);
      sink.addTestSuite(getElementsByTagNameNS08.class);
      sink.addTestSuite(getElementsByTagNameNS09.class);
      sink.addTestSuite(getElementsByTagNameNS10.class);
      sink.addTestSuite(getElementsByTagNameNS11.class);
      sink.addTestSuite(getElementsByTagNameNS12.class);
      sink.addTestSuite(getElementsByTagNameNS13.class);
      sink.addTestSuite(getElementsByTagNameNS14.class);
      sink.addTestSuite(getNamedItemNS01.class);
      sink.addTestSuite(getNamedItemNS02.class);
   //  sink.addTestSuite(getNamedItemNS03.class);
    //  sink.addTestSuite(getNamedItemNS04.class);
      sink.addTestSuite(hasAttribute01.class);
      sink.addTestSuite(hasAttribute02.class);
      sink.addTestSuite(hasAttribute03.class);
      sink.addTestSuite(hasAttribute04.class);
      sink.addTestSuite(hasAttributeNS01.class);
      sink.addTestSuite(hasAttributeNS02.class);
      sink.addTestSuite(hasAttributeNS03.class);
     // sink.addTestSuite(hasAttributeNS04.class);
      sink.addTestSuite(hasAttributeNS05.class);
      sink.addTestSuite(hasAttributes01.class);
      sink.addTestSuite(hasAttributes02.class);
      sink.addTestSuite(hc_attrcreatedocumentfragment.class);
      sink.addTestSuite(hc_attrname.class);
      sink.addTestSuite(hc_documentcreateattribute.class);
      sink.addTestSuite(hc_elementgetattributenode.class);
  //    sink.addTestSuite(hc_entitiesremovenameditemns1.class);
   //   sink.addTestSuite(hc_entitiessetnameditemns1.class);
      sink.addTestSuite(hc_namednodemapgetnameditem.class);
      sink.addTestSuite(hc_nodedocumentfragmentnormalize1.class);
      sink.addTestSuite(hc_nodedocumentfragmentnormalize2.class);
   //   sink.addTestSuite(hc_notationsremovenameditemns1.class);
 //     sink.addTestSuite(hc_notationssetnameditemns1.class);
  //    sink.addTestSuite(ibmtests.class);
   //   sink.addTestSuite(importNode01.class);
    //  sink.addTestSuite(importNode02.class);
      sink.addTestSuite(importNode03.class);
      sink.addTestSuite(importNode04.class);
  //    sink.addTestSuite(importNode05.class);
   //   sink.addTestSuite(importNode06.class);
      sink.addTestSuite(importNode07.class);
      sink.addTestSuite(importNode08.class);
    //  sink.addTestSuite(importNode09.class);
    //  sink.addTestSuite(importNode10.class);
    //  sink.addTestSuite(importNode11.class);
    //  sink.addTestSuite(importNode12.class);
     // sink.addTestSuite(importNode13.class);
      sink.addTestSuite(importNode14.class);
      sink.addTestSuite(importNode15.class);
     // sink.addTestSuite(importNode16.class);
      sink.addTestSuite(importNode17.class);
     // sink.addTestSuite(internalSubset01.class);
     sink.addTestSuite(isSupported01.class);
      sink.addTestSuite(isSupported02.class);
      sink.addTestSuite(isSupported04.class);
      sink.addTestSuite(isSupported05.class);
      sink.addTestSuite(isSupported06.class);
      sink.addTestSuite(isSupported07.class);
      sink.addTestSuite(isSupported09.class);
      sink.addTestSuite(isSupported10.class);
      sink.addTestSuite(isSupported11.class);
      sink.addTestSuite(isSupported12.class);
      sink.addTestSuite(isSupported13.class);
      sink.addTestSuite(isSupported14.class);
      sink.addTestSuite(localName01.class);
      sink.addTestSuite(localName02.class);
      sink.addTestSuite(localName03.class);
      sink.addTestSuite(localName04.class);
     // sink.addTestSuite(namednodemapgetnameditemns01.class);
      sink.addTestSuite(namednodemapgetnameditemns02.class);
      sink.addTestSuite(namednodemapgetnameditemns03.class);
      sink.addTestSuite(namednodemapgetnameditemns04.class);
      sink.addTestSuite(namednodemapgetnameditemns05.class);
      sink.addTestSuite(namednodemapgetnameditemns06.class);
      sink.addTestSuite(namednodemapremovenameditemns01.class);
       //default Attr
     // sink.addTestSuite(namednodemapremovenameditemns02.class);
     // ER
       //  sink.addTestSuite(namednodemapremovenameditemns03.class);
      sink.addTestSuite(namednodemapremovenameditemns04.class);
     // sink.addTestSuite(namednodemapremovenameditemns05.class);
      sink.addTestSuite(namednodemapremovenameditemns06.class);
      sink.addTestSuite(namednodemapremovenameditemns07.class);
      sink.addTestSuite(namednodemapremovenameditemns08.class);
      sink.addTestSuite(namednodemapremovenameditemns09.class);
      sink.addTestSuite(namednodemapsetnameditemns01.class);
      sink.addTestSuite(namednodemapsetnameditemns02.class);
      sink.addTestSuite(namednodemapsetnameditemns03.class);
      sink.addTestSuite(namednodemapsetnameditemns04.class);
    //  sink.addTestSuite(namednodemapsetnameditemns05.class);
      sink.addTestSuite(namednodemapsetnameditemns06.class);
      sink.addTestSuite(namednodemapsetnameditemns07.class);
      sink.addTestSuite(namednodemapsetnameditemns08.class);
    //  sink.addTestSuite(namednodemapsetnameditemns09.class);
     // sink.addTestSuite(namednodemapsetnameditemns10.class);
     // sink.addTestSuite(namednodemapsetnameditemns11.class);
      sink.addTestSuite(namespaceURI01.class);
      sink.addTestSuite(namespaceURI02.class);
      sink.addTestSuite(namespaceURI03.class);
      sink.addTestSuite(namespaceURI04.class);
      sink.addTestSuite(nodegetlocalname03.class);
      sink.addTestSuite(nodegetnamespaceuri03.class);
   //   sink.addTestSuite(nodegetownerdocument01.class);
      sink.addTestSuite(nodegetownerdocument02.class);
      sink.addTestSuite(nodegetprefix03.class);
      sink.addTestSuite(nodehasattributes01.class);
    //  sink.addTestSuite(nodehasattributes02.class);
      sink.addTestSuite(nodehasattributes03.class);
      sink.addTestSuite(nodehasattributes04.class);
    //  sink.addTestSuite(nodeissupported01.class);
     // sink.addTestSuite(nodeissupported02.class);
     // sink.addTestSuite(nodeissupported03.class);
     // sink.addTestSuite(nodeissupported04.class);
      // sink.addTestSuite(nodeissupported05.class);
     // sink.addTestSuite(nodenormalize01.class);
      sink.addTestSuite(nodesetprefix01.class);
      sink.addTestSuite(nodesetprefix02.class);
      sink.addTestSuite(nodesetprefix03.class);
     // sink.addTestSuite(nodesetprefix04.class);
      sink.addTestSuite(nodesetprefix05.class);
      sink.addTestSuite(nodesetprefix06.class);
      sink.addTestSuite(nodesetprefix07.class);
      sink.addTestSuite(nodesetprefix08.class);
      sink.addTestSuite(nodesetprefix09.class);
      sink.addTestSuite(normalize01.class);
      sink.addTestSuite(ownerDocument01.class);
      sink.addTestSuite(ownerElement01.class);
      sink.addTestSuite(ownerElement02.class);
      sink.addTestSuite(prefix01.class);
      sink.addTestSuite(prefix02.class);
      sink.addTestSuite(prefix03.class);
      sink.addTestSuite(prefix04.class);
      sink.addTestSuite(prefix05.class);
      sink.addTestSuite(prefix06.class);
      sink.addTestSuite(prefix07.class);
    //  sink.addTestSuite(prefix08.class);
      sink.addTestSuite(prefix09.class);
      sink.addTestSuite(prefix10.class);
      sink.addTestSuite(prefix11.class);
    //  sink.addTestSuite(publicId01.class);
    //  sink.addTestSuite(removeAttributeNS01.class);
   //   sink.addTestSuite(removeAttributeNS02.class);
      sink.addTestSuite(removeNamedItemNS01.class);
      sink.addTestSuite(removeNamedItemNS02.class);
      sink.addTestSuite(removeNamedItemNS03.class);
      sink.addTestSuite(setAttributeNS01.class);
      sink.addTestSuite(setAttributeNS02.class);
      //Test makes no sense w/o ER
    //  sink.addTestSuite(setAttributeNS03.class);
      sink.addTestSuite(setAttributeNS04.class);
      sink.addTestSuite(setAttributeNS05.class);
      sink.addTestSuite(setAttributeNS06.class);
      sink.addTestSuite(setAttributeNS07.class);
      sink.addTestSuite(setAttributeNS09.class);
      sink.addTestSuite(setAttributeNodeNS01.class);
          //Test makes no sense w/o ER
     // sink.addTestSuite(setAttributeNodeNS02.class);
      sink.addTestSuite(setAttributeNodeNS03.class);
      sink.addTestSuite(setAttributeNodeNS04.class);
      sink.addTestSuite(setAttributeNodeNS05.class);
      sink.addTestSuite(setNamedItemNS01.class);
      sink.addTestSuite(setNamedItemNS02.class);
      sink.addTestSuite(setNamedItemNS03.class);
         //Test makes no sense w/o ER
    //  sink.addTestSuite(setNamedItemNS04.class);
      sink.addTestSuite(setNamedItemNS05.class);
  //    sink.addTestSuite(systemId01.class);

      return sink;

   }

   /**
    *  Gets URI that identifies the test suite
    *  @return uri identifier of test suite
    */
   public String getTargetURI() {
      return "http://www.w3.org/2001/DOM-Test-Suite/level2/core/alltests";
   }


}
