sh ./v1.sh POReadAllV1 1 
sh ./v1.sh POReadAllV1 2 
sh ./v1.sh POReadAllV1 3 
sh ./v1.sh POReadAllV1 4 
sh ./v1.sh POReadAllV1 5 
sh ./v1.sh POReadAllV1 6 
sh ./v1.sh POReadAllV1 7 
sh ./v1.sh POReadOneV1 1
sh ./v1.sh POReadOneV1 2 
sh ./v1.sh POReadOneV1 3
sh ./v1.sh POReadOneV1 4
sh ./v1.sh POReadOneV1 5 
sh ./v1.sh POReadOneV1 6 
sh ./v1.sh POReadOneV1 7 
sh ./v1.sh POGetCustNameV1 filename 1
sh ./v1.sh POGetCustNameV1 filename 2
sh ./v1.sh POGetCustNameV1 filename 3
sh ./v1.sh POGetCustNameV1 filename 4
sh ./v1.sh POGetCustNameV1 filename 5
sh ./v1.sh POGetCustNameV1 filename 6
sh ./v1.sh POGetCustNameV1 filename 7
sh ./v1.sh POGetSetGetCustNameV1 filename 1
sh ./v1.sh POGetSetGetCustNameV1 filename 2
sh ./v1.sh POGetSetGetCustNameV1 filename 3
sh ./v1.sh POGetSetGetCustNameV1 filename 4
sh ./v1.sh POGetSetGetCustNameV1 filename 5
sh ./v1.sh POGetSetGetCustNameV1 filename 6
sh ./v1.sh POGetSetGetCustNameV1 filename 7
sh ./v1.sh POSelectLineItemsV1 filename 1
sh ./v1.sh POSelectLineItemsV1 filename 2
sh ./v1.sh POSelectLineItemsV1 filename 3
sh ./v1.sh POSelectLineItemsV1 filename 4
sh ./v1.sh POSelectLineItemsV1 filename 5
sh ./v1.sh POSelectLineItemsV1 filename 6
sh ./v1.sh POSelectLineItemsV1 filename 7
sh ./v1.sh POTopDownV1
sh ./v1.sh POTopDownSaveV1
sh ./v1.sh POValidateXmlObjV1
sh ./v1.sh PrimTopDownV1
sh ./v1.sh NPrimTopDownV1
sh ./v1.sh CursorWalkV1 deep-attributes
sh ./v1.sh CursorWalkV1 deep-elements
sh ./v1.sh CursorWalkV1 wide-attributes
sh ./v1.sh CursorWalkV1 wide-elements
sh ./v1.sh CursorWalkV1 wide-text
sh ./v1.sh CursorGetAttributeV1 deep-attributes
sh ./v1.sh CursorGetAttributeV1 wide-attributes
sh ./v1.sh CursorGetSetGetAttributeV1 deep-attributes
sh ./v1.sh CursorGetSetGetAttributeV1 wide-attributes
sh ./v1.sh CursorGetElementV1
sh ./v1.sh CursorGetSetGetElementV1
sh ./v1.sh BisLoadV1 deep-attributes
sh ./v1.sh BisLoadV1 deep-elements
sh ./v1.sh BisLoadV1 wide-attributes
sh ./v1.sh BisLoadV1 wide-elements
sh ./v1.sh BisLoadV1 wide-text
sh ./v1.sh DomWalkV1 deep-attributes
sh ./v1.sh DomWalkV1 deep-elements
sh ./v1.sh DomWalkV1 wide-attributes
sh ./v1.sh DomWalkV1 wide-elements 
sh ./v1.sh DomWalkV1 wide-text
sh ./v1.sh DomWalkNLV1 deep-attributes
sh ./v1.sh DomWalkNLV1 deep-elements
sh ./v1.sh DomWalkNLV1 wide-attributes
sh ./v1.sh DomWalkNLV1 wide-elements 
sh ./v1.sh DomWalkNLV1 wide-text
sh v1.sh MaxMemUntypedBisLoadV1 memory 64 flavor deep-attributes
sh v1.sh MaxMemUntypedBisLoadV1 memory 128 flavor deep-attributes
sh v1.sh MaxMemUntypedBisLoadV1 memory 256 flavor deep-attributes
sh v1.sh MaxMemUntypedBisLoadV1 memory 512 flavor deep-attributes
sh v1.sh MaxMemUntypedBisLoadV1 memory 64 flavor deep-elements
sh v1.sh MaxMemUntypedBisLoadV1 memory 128 flavor deep-elements
sh v1.sh MaxMemUntypedBisLoadV1 memory 256 flavor deep-elements
sh v1.sh MaxMemUntypedBisLoadV1 memory 512 flavor deep-elements
sh v1.sh MaxMemUntypedBisLoadV1 memory 64 flavor wide-attributes
sh v1.sh MaxMemUntypedBisLoadV1 memory 128 flavor wide-attributes
sh v1.sh MaxMemUntypedBisLoadV1 memory 256 flavor wide-attributes
sh v1.sh MaxMemUntypedBisLoadV1 memory 512 flavor wide-attributes
sh v1.sh MaxMemUntypedBisLoadV1 memory 64 flavor wide-text
sh v1.sh MaxMemUntypedBisLoadV1 memory 128 flavor wide-text
sh v1.sh MaxMemUntypedBisLoadV1 memory 256 flavor wide-text
sh v1.sh MaxMemUntypedBisLoadV1 memory 512 flavor wide-text
sh v1.sh MaxMemUntypedBisLoadV1 memory 64 flavor wide-text
sh v1.sh MaxMemUntypedBisLoadV1 memory 128 flavor wide-elements
sh v1.sh MaxMemUntypedBisLoadV1 memory 256 flavor wide-elements
sh v1.sh MaxMemUntypedBisLoadV1 memory 512 flavor wide-elements
