sh ./jibx.sh POReadAllJibx 1 
sh ./jibx.sh POReadAllJibx 2 
sh ./jibx.sh POReadAllJibx 3 
sh ./jibx.sh POReadAllJibx 4 
sh ./jibx.sh POReadAllJibx 5 
sh ./jibx.sh POReadAllJibx 6 
sh ./jibx.sh POReadAllJibx 7 
sh ./jibx.sh POReadOneJibx 1 
sh ./jibx.sh POReadOneJibx 2 
sh ./jibx.sh POReadOneJibx 3 
sh ./jibx.sh POReadOneJibx 4 
sh ./jibx.sh POReadOneJibx 5 
sh ./jibx.sh POReadOneJibx 6 
sh ./jibx.sh POReadOneJibx 7 
