call castor.bat POReadAllCastor filename 1 
call castor.bat POReadAllCastor filename 2 
call castor.bat POReadAllCastor filename 3 
call castor.bat POReadAllCastor filename 4 
call castor.bat POReadAllCastor filename 5 
call castor.bat POReadAllCastor filename 6 
call castor.bat POReadAllCastor filename 7 
call castor.bat POReadOneCastor filename 1
call castor.bat POReadOneCastor filename 2
call castor.bat POReadOneCastor filename 3
call castor.bat POReadOneCastor filename 4
call castor.bat POReadOneCastor filename 5
call castor.bat POReadOneCastor filename 6
call castor.bat POReadOneCastor filename 7
call castor.bat POGetCustNameCastor filename 1
call castor.bat POGetCustNameCastor filename 2
call castor.bat POGetCustNameCastor filename 3
call castor.bat POGetCustNameCastor filename 4
call castor.bat POGetCustNameCastor filename 5
call castor.bat POGetCustNameCastor filename 6
call castor.bat POGetCustNameCastor filename 7
call castor.bat POGetSetGetCustNameCastor filename 1
call castor.bat POGetSetGetCustNameCastor filename 2
call castor.bat POGetSetGetCustNameCastor filename 3
call castor.bat POGetSetGetCustNameCastor filename 4
call castor.bat POGetSetGetCustNameCastor filename 5
call castor.bat POGetSetGetCustNameCastor filename 6
call castor.bat POGetSetGetCustNameCastor filename 7
call castor.bat POTopDownCastor
call castor.bat POTopDownSaveCastor
call castor.bat PrimTopDownCastor
call castor.bat NPrimTopDownCastor
