sh ./v2.sh POReadAllV2 1 
sh ./v2.sh POReadAllV2 2 
sh ./v2.sh POReadAllV2 3 
sh ./v2.sh POReadAllV2 4 
sh ./v2.sh POReadAllV2 5 
sh ./v2.sh POReadAllV2 6 
sh ./v2.sh POReadAllV2 7 
sh ./v2.sh POReadOneV2 1
sh ./v2.sh POReadOneV2 2 
sh ./v2.sh POReadOneV2 3
sh ./v2.sh POReadOneV2 4
sh ./v2.sh POReadOneV2 5 
sh ./v2.sh POReadOneV2 6 
sh ./v2.sh POReadOneV2 7 
sh ./v2.sh POGetCustNameV2 filename 1
sh ./v2.sh POGetCustNameV2 filename 2
sh ./v2.sh POGetCustNameV2 filename 3
sh ./v2.sh POGetCustNameV2 filename 4
sh ./v2.sh POGetCustNameV2 filename 5
sh ./v2.sh POGetCustNameV2 filename 6
sh ./v2.sh POGetCustNameV2 filename 7
sh ./v2.sh POGetSetGetCustNameV2 filename 1
sh ./v2.sh POGetSetGetCustNameV2 filename 2
sh ./v2.sh POGetSetGetCustNameV2 filename 3
sh ./v2.sh POGetSetGetCustNameV2 filename 4
sh ./v2.sh POGetSetGetCustNameV2 filename 5
sh ./v2.sh POGetSetGetCustNameV2 filename 6
sh ./v2.sh POGetSetGetCustNameV2 filename 7
sh ./v2.sh POSelectLineItemsV2 filename 1
sh ./v2.sh POSelectLineItemsV2 filename 2
sh ./v2.sh POSelectLineItemsV2 filename 3
sh ./v2.sh POSelectLineItemsV2 filename 4
sh ./v2.sh POSelectLineItemsV2 filename 5
sh ./v2.sh POSelectLineItemsV2 filename 6
sh ./v2.sh POSelectLineItemsV2 filename 7
sh ./v2.sh POQueryLineItemsV2 filename 1
sh ./v2.sh POQueryLineItemsV2 filename 2
sh ./v2.sh POQueryLineItemsV2 filename 3
sh ./v2.sh POQueryLineItemsV2 filename 4
sh ./v2.sh POQueryLineItemsV2 filename 5
sh ./v2.sh POQueryLineItemsV2 filename 6
sh ./v2.sh POQueryLineItemsV2 filename 7
sh ./v2.sh POTopDownV2
sh ./v2.sh POTopDownSaveV2
sh ./v2.sh POValidateXmlObjV2
sh ./v2.sh PrimTopDownV2
sh ./v2.sh NPrimTopDownV2
sh ./v2.sh CursorWalkV2 flavor deep-attributes
sh ./v2.sh CursorWalkV2 flavor deep-elements
sh ./v2.sh CursorWalkV2 flavor wide-attributes
sh ./v2.sh CursorWalkV2 flavor wide-elements
sh ./v2.sh CursorWalkV2 flavor wide-text
sh ./v2.sh CursorGetAttributeV2 flavor deep-attributes
sh ./v2.sh CursorGetAttributeV2 flavor wide-attributes
sh ./v2.sh CursorGetSetGetAttributeV2 flavor deep-attributes
sh ./v2.sh CursorGetSetGetAttributeV2 flavor wide-attributes
sh ./v2.sh CursorGetElementV2
sh ./v2.sh CursorGetSetGetElementV2
sh ./v2.sh BisLoadV2 flavor deep-attributes
sh ./v2.sh BisLoadV2 flavor deep-elements
sh ./v2.sh BisLoadV2 flavor wide-attributes
sh ./v2.sh BisLoadV2 flavor wide-elements
sh ./v2.sh BisLoadV2 flavor wide-text
sh ./v2.sh DomWalkV2 flavor deep-attributes
sh ./v2.sh DomWalkV2 flavor deep-elements
sh ./v2.sh DomWalkV2 flavor wide-attributes
sh ./v2.sh DomWalkV2 flavor wide-elements 
sh ./v2.sh DomWalkV2 flavor wide-text
sh ./v2.sh DomWalkNLV2 flavor deep-attributes
sh ./v2.sh DomWalkNLV2 flavor deep-elements
sh ./v2.sh DomWalkNLV2 flavor wide-attributes
sh ./v2.sh DomWalkNLV2 flavor wide-elements 
sh ./v2.sh DomWalkNLV2 flavor wide-text
sh v2.sh MaxMemUntypedBisLoadV2 memory 64 flavor deep-attributes
sh v2.sh MaxMemUntypedBisLoadV2 memory 128 flavor deep-attributes
sh v2.sh MaxMemUntypedBisLoadV2 memory 256 flavor deep-attributes
sh v2.sh MaxMemUntypedBisLoadV2 memory 512 flavor deep-attributes
sh v2.sh MaxMemUntypedBisLoadV2 memory 64 flavor deep-elements
sh v2.sh MaxMemUntypedBisLoadV2 memory 128 flavor deep-elements
sh v2.sh MaxMemUntypedBisLoadV2 memory 256 flavor deep-elements
sh v2.sh MaxMemUntypedBisLoadV2 memory 512 flavor deep-elements
sh v2.sh MaxMemUntypedBisLoadV2 memory 64 flavor wide-attributes
sh v2.sh MaxMemUntypedBisLoadV2 memory 128 flavor wide-attributes
sh v2.sh MaxMemUntypedBisLoadV2 memory 256 flavor wide-attributes
sh v2.sh MaxMemUntypedBisLoadV2 memory 512 flavor wide-attributes
sh v2.sh MaxMemUntypedBisLoadV2 memory 64 flavor wide-elements
sh v2.sh MaxMemUntypedBisLoadV2 memory 128 flavor wide-elements
sh v2.sh MaxMemUntypedBisLoadV2 memory 256 flavor wide-elements
sh v2.sh MaxMemUntypedBisLoadV2 memory 512 flavor wide-elements
sh v2.sh MaxMemUntypedBisLoadV2 memory 64 flavor wide-text
sh v2.sh MaxMemUntypedBisLoadV2 memory 128 flavor wide-text
sh v2.sh MaxMemUntypedBisLoadV2 memory 256 flavor wide-text
sh v2.sh MaxMemUntypedBisLoadV2 memory 512 flavor wide-text
